#!/bin/sh
#
# Simple script to find rpm packages that does not provide any resources
# required by other packges. 
# Released into the public domain. No restrictions.
# - christht - 
#
PATH="/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/bin"

#
# Check for needed binaries:
#
rpm --version >/dev/null 2>&1
if [ "$?" != "0" ]; then
    echo "Error: rpm binary not found!"
    exit 127
fi
rpm -q mktemp >/dev/null 2>&1
if [ "$?" != "0" ]; then
    echo "Error: mktemp binary not found!"
    exit 127
fi
sort --version >/dev/null 2>&1
if [ "$?" != "0" ]; then
    echo "Error: sort binary not found!"
    exit 127
fi
cat --version >/dev/null 2>&1
if [ "$?" != "0" ]; then
    echo "Error: cat binary not found!"
    exit 127
fi
rm --version >/dev/null 2>&1
if [ "$?" != "0" ]; then
    echo "Error: rm binary not found!"
    exit 127
fi



CTMP=$(mktemp /tmp/cleanfile.XXXXXX)
INDEX=0
echo "Calculating dependencies:"
for i in $(rpm -qa --queryformat %{name}-%{version}-%{release}' '); do 
    FOUND=0
    for j in $i `rpm -q --provides $i`; do 
        rpm -q --whatrequires $j >/dev/null
        if [ "$?" == "0" ]; then 
            FOUND=1 
            break; 
        fi
    done
    if [ FOUND == 0 ]; then
        for j in `rpm -ql $i`; do 
            rpm -q --whatrequires $j >/dev/null
            if [ "$?" == "0" ]; then 
                FOUND=1 
                break; 
            fi
        done
    fi
    if [ $FOUND == 0 ]; then
        echo -n "."; 
        echo $i >> $CTMP;
        INDEX=$(($INDEX+1))
    fi
done
echo ""
echo "The following packages does not provide any resources required by any"
echo "other packge installed on this system:"
cat $CTMP | sort
echo "Total $INDEX packages."
rm -rf $CTMP
