#!/bin/sh
#
# Script to update the MBR with correct information after a kernel 
# installation / upgrade
#
# Who wrote this hack?
# 
# v0.1.5    2005-11-10, Ajith Thampi	
# v0.1.4    2005-07-27, Syed Shabir Zakiullah	
# v0.1.3    2005-05-18, Syed Shabir Zakiullah	 	
# v0.1.2    2003-06-16, christht
# v0.1.1    2003-06-13, christht
# v0.1.0    2003-06-03, christht
# v0.0.9    2003-05-20, christht
# v0.0.8    2003-05-20, christht
# v0.0.7    2003-05-20, christht
# v0.0.6    2003-05-02, christht
# v0.0.5    2003-04-24, christht
# v0.0.4    2003-04-23, christht
# v0.0.3    2003-04-23, christht
# v0.0.2    2003-04-22, christht
# v0.0.1    2003-04-07, christht
#
#
# It reads configuration variables from the file:
#   /etc/sysconfig/bootloader
# or whatever is defined by $CONFIG
#
# The following variables are used:
#   BOOTLOADER=[grub,lilo,none]
#   INITRD=[yes,no]
#   AUTOACTIVATE=[yes,no]
#   REBOOT=[yes,no]
#   REBOOTAT=(see shutdown manpage for syntax)
#   MAILTO=
#
#



#
# Some functions:
#


usage() {
# check number of arguments
    echo "Wrong number of arguments, exiting!"
    echo "Usage:"
    echo "$0 [--mkinitrd] [--title=<title>] <--install | --remove> <kernel-version>"
    exit 1

}


function mailto(){
if [ "$MAILTO" != "" ]; then
    if [ -f "$MAILFILE" ]; then
	if [ -x /bin/hostname ]; then
	    HOSTNAME=$(/bin/hostname)
	    SUBJ="Kernel upgrade on '$HOSTNAME'"
	else
	    SUBJ='Kernel upgrade'
	fi
	    cat $MAILFILE | mail -s "$SUBJ" $MAILTO > /dev/null 2>&1
    else
	echo "Could not find mail content: $MAILFILE"
    fi
fi
rm -f $MAILFILE >/dev/null 2>&1 || :
}

function abandon(){
mailto
exit 1
}


make_initrd() {
  	#
        # create initrd, using arguments.
        # If this turns out broken, break all.
        #

        MKOUTPUT=$(mkinitrd -o $DIR/$IMG $KVER 2>&1)
        if [ "$?" != 0 ]; then
                echo -e "Errors when creating initrd:\n\n$MKOUTPUT\n" \
                | tee -a $MAILFILE
                abandon
        fi

        #
        # Simple test to see if initrd _looks_ ok:
        #
        pushd $DIR >/dev/null 2>&1
        OK=$(file $IMG | grep -q "Linux Compressed ROM"; echo $?)
        if [ ! "$OK" = "0" ]; then
                echo "initrd does not look valid. Exiting!" \
                | tee -a $MAILFILE
                abandon
        fi
        popd >/dev/null 2>&1

}

install(){

       	#
	# Check whether kernel for $KVER exists or not
	#
	if [ ! -f "$DIR/$VML" ]; then
		echo "Kernel for $KVER not present, not running grubby"
		return
	fi

	#
       	# Collecting Root device information
       	#	
       	rootdevice=$(awk '{ if ($1 !~ /^[ \t]*#/ && $2 == "/") { print $1; }}' /etc/fstab)
        
        #
        # Common arguments
        # 
        opts="--add-kernel=$DIR/$VML --copy-default"
	args="root=$rootdevice"
        
        #
	# this kernel needs an initrd
	#
	if [ "$INITRD" = "yes" ]; then
		opts="$opts --initrd=$DIR/$IMG"
		args="$args ramdisk_size=8192"
	fi

	#
	# Creating default entry 	
	#
	if [ "$1" = "yes" ]; then
		opts="$opts --make-default"
	else
		echo "Not configured to autoactivate new kernel." \
		| tee -a $MAILFILE
		mailto
	fi

	#
	# Bootloader selection
	#
	if [ "$BOOTLOADER" = "lilo" ]; then
		opts="$opts --lilo"
	elif [ "$BOOTLOADER" = "grub" ]; then
		opts="$opts --grub"
		args="$args ro"
	else
		echo "No valid booloader" | tee -a $MAILFILE
	fi

	/sbin/grubby $opts --args="$args" --title="$2" --remove-kernel="TITLE=$2"

}

remove() {
	#
	# Removing entries from conf files
        #

	if [ "$BOOTLOADER" = "lilo" ]; then
       	        /sbin/grubby --remove-kernel=$DIR/$VML --lilo
        elif [ "$BOOTLOADER" = "grub" ]; then
                /sbin/grubby --remove-kernel=$DIR/$VML --grub
       	else echo "No valid bootloader" \
               	| tee -a $MAILFILE

        fi

	#
	# Remove initrd from $DIR
	#
	if [ -f "$DIR/$IMG" ]; then
		rm -rf $DIR/$IMG
	fi
	
	exit 0

}

while [ $# -gt 0 ]; do
    case $1 in
        
	--mkinitrd)
	    makeinitrd=$1
	    ;; 	
	--title*)
	    if echo $1 | grep '=' >/dev/null ; then
	    	title=`echo $1 | sed 's/^--title=//'` 	
	    else
		title=$2
		shift
	    fi	
	    ;; 	
	--install)
            install="yes"
	    mode_install=$1
	    ;;
        --remove)
	    remove="yes"	
	    mode_remove=$1
	    ;;
	 *)
            if [ -z "$KVER" ]; then
                KVER=$1
	    else
                usage
            fi
            ;;
    esac
shift
done

if [ "$mode_install" != "--install" -a "$mode_remove" != "--remove" ] ; then
	if [ ! "$makeinitrd" = "--mkinitrd" ]; then
		usage
	fi
fi

if [ -z "$KVER" ]; then
    usage
fi

if [ "$mode_install" = "--install" -a "$mode_remove" = "--remove" ]; then
    usage
fi

#
# A file to add content of possible mail:
#
MAILFILE=$(mktemp /tmp/kernmail.XXXXXX)

#
# Setting default variables:
# ( Paranoid, do nothing! )
# 
CONFIG='/etc/sysconfig/bootloader'
BOOTLOADER=none
INITRD=yes
AUTOACTIVATE=yes
REBOOT=no
REBOOTAT=+60
MAILTO=


#
# Get values from configuration file:
#
if [ -f "$CONFIG" ]; then
    . $CONFIG
else
    echo "File '$CONFIG' not found, using defaults." \
	| tee -a $MAILFILE
fi
    
DIR='/boot'
IMG="initrd-$KVER.img"
VML="vmlinuz-$KVER"


if [ -z "$title" ]; then
        if [ "$BOOTLOADER" = "grub" ]; then
        # Set Default to Trustix Secure Linux
        title="Trustix Secure Linux ($KVER)"
        elif [ "$BOOTLOADER" = "lilo" ]; then
        # Set Default to Trustix Secure Linux
        title="Trustix"
        fi
fi

if [ "$makeinitrd" = "--mkinitrd" -a "$INITRD" = "yes" ]; then
	make_initrd $KVER
fi

if [ "$install" = "yes" ]; then
	install "$AUTOACTIVATE" "$title"
fi

if [ "$remove" = "yes" ]; then
	remove
fi

#
# We may want to update MBR since AUTOACTIVATE='yes'
#
if [ "$BOOTLOADER" = "lilo" -a -x /sbin/lilo ]; then
      /sbin/lilo 2>&1 > /dev/null
elif [ "$BOOTLOADER" != "grub" ]; then
      echo "No valid bootloader set. Not updating MBR." \
	| tee -a $MAILFILE
fi
  
#
# Are we going to reboot?
#
if [ "$REBOOT" = "yes" -a ! -z "$REBOOTAT" ]; then
    # shutdown only if $REBOOTAT != [now,0] inn background
    if [ "$REBOOTAT" != "now" -a "$REBOOTAT" != "+0" ]; then
        shutdown -r $REBOOTAT "Rebooting due to kernel upgrade."  & \
	    > /dev/null 2>&1
        echo "" | tee -a $MAILFILE
        echo "Rebooting due to kernel upgrade. " \
	    | tee -a $MAILFILE
	echo "New kernel version is: '$KVER'" \
	    | tee -a $MAILFILE
        echo "" | tee -a $MAILFILE
	echo "The time argument given to shutdown was: '$REBOOTAT'" \
	    | tee -a $MAILFILE
        echo "Use 'shutdown -c' to cancel shutdown." \
	    | tee -a $MAILFILE
    else
        echo "To reboot _now_ (during upgrade) is probably not a good idea.."\
	    | tee -a $MAILFILE
        echo "Please re-evaluate your 'REBOOTAT' setting."\
	    | tee -a $MAILFILE
    fi
else
    echo "Will not reboot." \
	| tee -a $MAILFILE
fi


# Done
#
mailto
exit 0
